/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <string.h>

#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/stat.h>

#include <linux/fd.h>

#include "sdgen.h"

#ifndef PACKAGE
#define PACKAGE	"fpdaemon"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) gettext (String)
#else
#define _(String) String
#endif

#ifdef HAVE_GNOME
#  include <gnome.h>
#  include "interface.h"
#  include "support.h"
#endif

#define PROG_NAME	PACKAGE

#define TM_CHECK_INTERVAL	1	/* Check for stuff every second */
#define TM_CHECK_STATUS		3	/* Check status every 3 seconds */

static const char *FPDpidfile=NULL;
#ifdef HAVE_GNOME
static int FPDcanwindow=0;		/* 1 if we can use the popups */
#endif
static int FPDdienow=0;			/* 1 if we should die now (set on signal) */

static struct fpd_drive {
	time_t	next_check;		/* Time when we check the drive next */
#ifdef HAVE_GNOME
	GtkWidget *popup;		/* Popup widget for interactive messages */
#endif
	SDERROR last_status;		/* Last status reported */
} fpd_drive[MAX_DRIVE];

/* Default drive status
 */
static struct fpd_drive fpd_drive_default;

static int check_drive(int drive)
{
	const char *mess,*solmess;
	char device[512];
	int fd,err;
	SDERROR status;
	struct stat st;
	time_t now;
	enum { REPORT_NONE=0,
		   REPORT_DEBUG,
		   REPORT_ERROR } report;
	enum { SOLN_NONE=0,
		SOLN_EJECT,
		SOLN_MEDIA_BAD } solution;

	if (FPDdienow) {
#ifdef HAVE_GNOME
		if (FPDcanwindow) {	/* In GTK+ */
			gtk_main_quit();
		} else {		/* Normal */
			exit(0);
		}
#else
		exit(0);
#endif
	}

	now=time(NULL);

	/* Not time to check it yet..
	 * (this allows for the time for the inode mangler to expire)
	 */
	if (now < fpd_drive[drive].next_check)
		return TRUE;

	fpd_drive[drive].next_check=now+TM_CHECK_STATUS;
	sprintf(device,"/dev/fd%d",drive);
	err=stat(device,&st);
	if (err)
		return TRUE;		/* No such device in /dev */

	if (! S_ISBLK(st.st_mode))
		return TRUE;		/* Not a block device */

	fd=open(device,O_RDONLY|O_NDELAY,0);	/* Magic ioctl-only open */
	if (fd < 0) 
		return TRUE;		/* Hmm. Can't open... */


	status=(SDERROR)ioctl(fd,FPSTATUS,0);		/* Check status */
	close(fd);

	if (status==-1)
		return TRUE;		/* No flashpath driver */

	switch (status) {
		case SDE_SUCCESS:
			mess=_("Device ok");
			report=REPORT_NONE;
			solution=SOLN_NONE;
			break;
		case SDE_FP_FIRMWARE_INVALID_DEVICE:
		case SDE_FP_INPUT_ERROR:
			mess=_("I/O error");
			report=REPORT_ERROR;
			solution=SOLN_EJECT;
			break;
		case SDE_MEDIA_CHANGE_DETECTED:
			mess=_("Media changed");
			report=REPORT_DEBUG;
			solution=SOLN_NONE;
			break;
		case SDE_SECTOR_NOT_FOUND:
			mess=_("Sector not found");
			report=REPORT_ERROR;
			solution=SOLN_EJECT;
			break;
		case SDE_DEVICE_POWERED_OFF:
			mess=_("The device has gone to sleep. Please reinsert");
			report=REPORT_ERROR;
			solution=SOLN_EJECT;
			break;
		case SDE_NO_MEDIA_IN_DEVICE:
			mess=_("No FlashPath media");
			report=REPORT_DEBUG;
			solution=SOLN_MEDIA_BAD;
			break;
		case SDE_MEDIA_WRITE_PROTECTED:
			mess=_("The media is write protected, or your batteries are low");
			report=REPORT_ERROR;
			solution=SOLN_NONE;
			break;
		case SDE_INVALID_DEVICE_REQUEST:
			mess=_("Invalid device request");
			report=REPORT_DEBUG;
			solution=SOLN_NONE;
			break;
		case SDE_CRC_ERROR:
			mess=_("CRC error - your batteries may be low, or your memory device is faulty");
			report=REPORT_ERROR;
			solution=SOLN_NONE;
			break;
		case SDE_NOT_SUPPORTED:
			mess=_("The device is not supported by this driver");
			solution=SOLN_NONE;
			report=REPORT_ERROR;
			break;
		case SDE_READ_BEYOND_END_OF_MEDIA:
			mess=_("Driver error: attempt to read beyond end of media");
			report=REPORT_ERROR;
			solution=SOLN_NONE;
			break;
		case SDE_INVALID_LOGICAL_FORMAT:
			mess=_("The media in the device is not formatted");
			report=REPORT_ERROR;
			solution=SOLN_MEDIA_BAD;
			break;
		default:
			mess=_("The device has reported an unusual fault");
			report=REPORT_ERROR;
			solution=SOLN_EJECT;
			break;
	}

	switch (solution) {
		case SOLN_NONE:
			solmess=NULL;
			break;			/* Nothing... */
		case SOLN_MEDIA_BAD:
			solmess=_("Your flash token may be bad, or improperly inserted.");
			break;
		case SOLN_EJECT:
			solmess=_("Please eject, wait a second, then reinsert you FlashPath device to try again.");
			break;
		default:
		   	fprintf(stderr,PROG_NAME ": ");
			fprintf(stderr,_("How in the heck did I get _HERE_?\n"));
		   	fprintf(stderr,PROG_NAME ": ");
			fprintf(stderr,_("Solution is set to %d, status=0x%X on device %s\n"),(int)solution,(int)status,device);
			return FALSE;
	}

	switch (report) {
		case REPORT_NONE:
			break;			/* Nothing... */
		case REPORT_DEBUG:
#ifdef DBG
			fprintf(stderr,PROG_NAME ": %s: %s [0x%X]\n",mess,device,status);
#endif
			break;
		case REPORT_ERROR:
#ifdef HAVE_GNOME
			if (FPDcanwindow) {		/* In a GUI... */
				GtkWidget *popup;

				popup=fpd_drive[drive].popup;
				if (GTK_IS_WIDGET(popup))
					gtk_widget_destroy(popup);

				popup=create_FPDpopup(mess,solmess,
							device,status);
				gtk_widget_show(popup);
				fpd_drive[drive].popup=popup;
			} else {				/* In a console... */
#else
			{
#endif
				fprintf(stderr,PROG_NAME ": %s: %s [0x%X]\n",
								mess,device,status);
				fprintf(stderr,PROG_NAME ": %s\n",solmess);
			}
			break;
		default:
		   	fprintf(stderr,PROG_NAME ": ");
			fprintf(stderr,_("How in the heck did I get _HERE_?\n"));
		   	fprintf(stderr,PROG_NAME ": ");
			fprintf(stderr,_("Report is set to %d, status=0x%X on device %s\n"),(int)report,(int)status,device);
			return FALSE;
	}

	return TRUE;
}

#ifdef HAVE_GNOME
static gint FPD_check_drive(gpointer data)
{
	int drive=data-NULL;

	return check_drive(drive);
}
#endif

static void signal_die(int sig)
{
	/* Hmm.. We're being killed.. Argh.
	 * Clean up the old pid & exit
	 */
	if (FPDpidfile != NULL)
		unlink(FPDpidfile);	

	FPDdienow=1;
}

static void killold(const char *path)
{
	FILE *pidf;
	char buff[1024];
	pid_t pid;
	int err;
	struct stat st;

	pidf=fopen(path,"r");
	if (pidf != NULL) {
		/* Hmmm.. Old pid file.. let's see if it's stale.
		 */
		fgets(buff,1024,pidf);
		fclose(pidf);
		pid=(pid_t)atoi(buff);
		if (pid > 0) {
			/* Valid pid... */
			kill(pid,SIGTERM);
			sleep(2);
			err=stat(path,&st);
			if (err) {
				if (errno != ENOENT) {
					fprintf(stderr,PROG_NAME ": ");
					fprintf(stderr,_("Please manually kill any old fpdaemon processed and\n"
									"remove the old %s pid file.\n"),path);
					exit(errno);
				}
				/* If we get here, the old fpdaemon cleaned up after itself.
				 */
				fprintf(stderr,"FlashPath Status Monitor v"VERSION": stopped (%d)\n",pid);
			} else {
				fprintf(stderr,PROG_NAME ": ");
				fprintf(stderr,_("pid %d in %s is stale. But I'll take care of it...\n"),pid,path);
				kill(pid,SIGKILL);
				unlink(path);
			}
		}
	}
	/* Ok. Now we (hopefully) have killed off any old fpdaemon processes
	 */
}

static void daemonize(const char *path)
{
	FILE *pidf;
	char buff[1024];
	pid_t pid;
	int err;

	/* Kill any old copies
	 */
	killold(path);

	/* We attempt to create the pid file here so
	 * that we don't have to try to kill the child if it fails.
	 */
	pidf=fopen(path,"w");
	if (pidf == NULL) {
		fprintf(stderr,PROG_NAME ": ");
		fprintf(stderr,_("Can't create pid status file %s:"),path);
		perror("");
		exit(errno);
	}

	/* Fork and create new pid file.
	 */
	pid=fork();
	if (pid<0) {
		fprintf(stderr,PROG_NAME ": ");
		fprintf(stderr,_("Couldn't fork a background process.\n"));
		exit(errno);
	}

	if (pid!=0) {
		/* Parent: I write the pid file.
		 */
		fprintf(pidf,"%d\n",pid);
		fclose(pidf);
		_exit(0);			/* We use _exit() so as not to disturb GTK+ */
	}

	/* Hey! I'm the child!
	 * Let's set up a signal handler so we can exit cleanly.
	 */
	fclose(pidf);
	FPDpidfile=strdup(path);

	signal(SIGTERM,signal_die);

	return;
}

int main(int argc,char **argv)
{
	int drive;
	int do_fork=1,do_kill=0;
	char buff[PATH_MAX];
	const char *home,*dpy;

#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	if (argc > 1 && strcmp(argv[1],"-V")==0) {
		fprintf(stderr,"FlashPath Status Monitor v"VERSION"\n");
		exit(0);
	}

	if (argc > 1 && strcmp(argv[1],"-k")==0) {
		do_kill=1;
		argc--;
		memcpy(argv+1,argv+2,sizeof(*argv)*(argc-1));
	}
		
	if (argc > 1 && strcmp(argv[1],"-d")==0) {
		do_fork=0;
		argc--;
		memcpy(argv+1,argv+2,sizeof(*argv)*(argc-1));
	}

#ifdef HAVE_GNOME
	/* If we have X11, enable the interactive popups.
	 * Otherwise, just print out the status.
	 */
	dpy=getenv("DISPLAY");
	if (dpy!=NULL) {
		gnome_init(PROG_NAME, VERSION, argc, argv);
		FPDcanwindow=1;
	}
#endif

	home=getenv("HOME");
	if (home==NULL) {
		fprintf(stderr,PROG_NAME ": ");
		fprintf(stderr,_("%s: You will need to define your HOME environment variable.\n"),argv[0]);
		exit(1);
	}

	sprintf(buff,"%s/." PROG_NAME ".pid",home);

	/* We just want to.. KILL... KILL... KILL...
	 */
	if (do_kill) {
		killold(buff);
		exit(0);
	}

	/* Fork ourselves, and make sure another copy
	 * of fpdaemon isn't running.
	 */
	if (do_fork)
		daemonize(buff);

	fprintf(stderr,"FlashPath Status Monitor v"VERSION": started (%d)\n",getpid());

	fpd_drive_default.next_check=0;
#ifdef HAVE_GNOME
	fpd_drive_default.popup=NULL;
#endif

	/* Set up the default status of the drives
	 */
	for (drive=0;drive<MAX_DRIVE;drive++)
		fpd_drive[drive]=fpd_drive_default;

#ifdef HAVE_GNOME
	if (FPDcanwindow) {
		/* GTK version */
		for (drive=0;drive<MAX_DRIVE;drive++) {
			gtk_timeout_add(TM_CHECK_INTERVAL*1000,
							FPD_check_drive,(((char *)NULL)+drive));
		}

		gtk_main();
		return 0;
	}
#endif

	/* Otherwise, we're in text mode. 
	 * Now check the drives until we're killed
	 */
	for (;;) {
		for (drive=0;drive<MAX_DRIVE;drive++) {
			check_drive(drive);
			sleep(TM_CHECK_INTERVAL);
		}
	}
}
