/* sdtrace.h - include file for trace debug class
 * (c) Copyright 1996 Vireo Software, Inc.
 *
 * $Log: SDTrace.h,v $
 * Revision 1.3  2000/04/14 13:49:16  gus
 * Corel fixes
 *
 * Revision 1.2  2000/01/27 22:32:24  jmcmulla
 * Initial Linux Port
 *
 *   
 *      Rev 1000.5   Dec 20 1999 17:38:46   larryplib
 *   12-16-1999 and 12-20-1999 changes from RJM and JG
 *  
 *     Rev 1000.4   Dec 02 1999 14:08:36   larryplib
 *  12/02/99 changes by RJM
 *  RJM 12/01/99 restored dropped changes and log lines
 *  
 *     Rev 1000.3   Nov 29 1999 18:00:52   larryplib
 *  FPSM V3.07.01 WIN9X GA 11-16-1999
 *     Rev 1000.2   Oct 14 1999 15:28:48   larryplib
 *  10/08/1999 merge changes from Benoit
 *
 *   Rev 2.4   Sep 27 1999 15:19:12   larryplib
 * 
 *
 *   Rev 2.3   Sep 20 1999 02:01:48   larryplib
 * 
 *
 *   Rev 2.2   Aug 10 1999 18:25:58   larryplib
 * 
 *
 * GMF 08/05/99 -- LINT-like changes for Linux
 *   Rev 2.1   Jun 17 1999 16:19:24   larryplib
 * 
 *
 *   Rev 2.0   May 21 1999 10:56:38   larryplib
 * 
 *
 *   Rev 1.4   May 21 1999 10:54:00   larryplib
 * 
 *
 *   Rev 1.3   May 21 1999 10:23:02   larryplib
 * 
 *
 *   Rev 1.2   May 20 1999 14:01:44   larryplib
 *       5/20/99 chgs by RJM and BL for FlashPath and FlashPath MS
 *
 * RJM 5/14/99 Added #pragma message's to show when SDTrace is active.
 * 		These are required to debug mismatch problems between SDDEV
 *		and other components.
 */

#if !defined(__SDTRACE__)
#define __SDTRACE__

#ifndef __TRACE__

	#define	InfoLevel0	0x00010000	/* TraceInfoCat1 */
	#define	InfoLevel1	0x00020000	/* TraceInfoCat2 */
	#define	InfoLevel2	0x00040000	/* TraceInfoCat3 */
	#define	InfoLevel3	0x00080000	/* TraceInfoCat4 */
	#define	InfoLevel4	0x00100000	/* TraceInfoCat5 */
	#define	InfoLevel5	0x00200000	/* TraceInfoCat6 */
	#define	InfoLevel6	0x00400000	/* TraceInfoCat7 */
	#define	InfoLevel7	0x00800000	/* TraceInfoCat8 */
	#define	InfoLevel8	0x01000000	/* TraceInfoCat9 */
	#define	InfoLevel9	0x02000000	/* TraceInfoCat10 */
	#define	InfoLevel10	0x04000000	/* TraceInfoCat11 */
	#define	InfoLevel11	0x08000000	/* TraceInfoCat12 */
	#define	InfoLevel12	0x10000000	/* TraceInfoCat13 */
	#define	InfoLevel13	0x20000000	/* TraceInfoCat14 */
	#define	InfoLevel14	0x40000000	/* TraceInfoCat15 */

	/* TraceInfoCat stands for Trace Info Category. */
	typedef enum
	{
		TraceAlways,
		TraceInfo,			/* Equivalent to TraceInfoCat0 */
		TraceWarning,
		TraceError,
		TraceFatal,
		TraceNever,
		TraceInfoCat1		=	TraceInfo | InfoLevel0,
		TraceInfoCat2		=	TraceInfo | InfoLevel1,
		TraceInfoCat3		=	TraceInfo | InfoLevel2,
		TraceInfoCat4		=	TraceInfo | InfoLevel3,
		TraceInfoCat5		=	TraceInfo | InfoLevel4,
		TraceInfoCat6		=	TraceInfo | InfoLevel5,
		TraceInfoCat7		=	TraceInfo | InfoLevel6,
		TraceInfoCat8		=	TraceInfo | InfoLevel7,
		TraceInfoCat9		=	TraceInfo | InfoLevel8,
		TraceInfoCat10		=	TraceInfo | InfoLevel9,
		TraceInfoCat11		=	TraceInfo | InfoLevel10,
		TraceInfoCat12		=	TraceInfo | InfoLevel11,
		TraceInfoCat13		=	TraceInfo | InfoLevel12,
		TraceInfoCat14		=	TraceInfo | InfoLevel13,
		TraceInfoCat15		=	TraceInfo | InfoLevel14 
	} TRACE_LEVEL;

	typedef enum
	{
		BreakAlways,
		BreakInfo,
		BreakWarning,
		BreakError,
		BreakFatal,
		BreakNever
	} BREAK_LEVEL;

	typedef enum
	{
		TraceHex,
		TraceDecimal
	} TRACE_RADIX;

#if defined(__cplusplus) && !defined(CPPFAKE)
	CONST ULONG TRACE_MONITOR	= 1;
	CONST ULONG TRACE_DEBUGGER	= 2;
	CONST ULONG TRACE_FILE		= 0x8000;	/* Chose 0x8000 in case Vireo adds some in between. */
#endif

#endif /* __TRACE__ */

#if defined(__cplusplus) && !defined(CPPFAKE)

CONST	ULONG OUTPUT_ATTRIB_ADD_CR	  =	0x80000000; // Output CR after each line
CONST	ULONG OUTPUT_ATTRIB_ADD_TIME  =	0x40000000; // Add timestamp to prefix
CONST	ULONG OUTPUT_ATTRIB_USE_INT3  =	0x20000000; // Use Int 3 for breakpoints instead of Int 1


#if defined(DISABLE_SDTRACE)
	#error Do not define DISABLE_SDTRACE - use class  SDDebugOnlyTrace
#endif

class SDTrace
{
	SAFE_DESTRUCTORS
public:
	SDTrace(
		PCHAR prefix = (PCHAR)NULL,
#if defined _X86_
		ULONG targetmask = TRACE_DEBUGGER,
#else	// !_X86_
		ULONG targetmask = TRACE_DEBUGGER|TRACE_MONITOR,
#endif	// _X86_
		TRACE_LEVEL outputlevel = TraceAlways,
#if defined( SDNT_DRVR_LIB)
		BREAK_LEVEL breaklevel = BreakNever,
		PUNICODE_STRING ChannelName = NULL,
		PKINTERRUPT intobject = NULL
#elif defined( SD_WIN32 )
		BREAK_LEVEL breaklevel = BreakNever,
		PCHAR filepath = NULL
#else	// !SD_WIN32
		BREAK_LEVEL breaklevel = BreakNever
#endif	// SDNT_DRVR_LIB
		);

#if defined( SDNT_DRVR_LIB )
	SDTrace(
		PUNICODE_STRING prefix,
	#if defined _X86_
		ULONG targetmask = TRACE_DEBUGGER,
	#else
		ULONG targetmask = TRACE_DEBUGGER|TRACE_MONITOR,
	#endif	// _X86_
		TRACE_LEVEL outputlevel = TraceAlways,
		BREAK_LEVEL breaklevel = BreakNever,
		PUNICODE_STRING ChannelName = NULL,
		PKINTERRUPT intobject = NULL
		);
#endif	// SDNT_DRVR_LIB

	SDTrace(SDTrace&);

	~SDTrace(void);

	SDTrace&  operator << (const char dbch);
	SDTrace&  operator << (const char* string);
	SDTrace&  operator << (const ULONG dwordValue);
	SDTrace&  operator << (const USHORT wordValue);
	SDTrace&  operator << (const UCHAR byteValue);
#if defined( SDNT_DRVR_LIB )
	SDTrace&  operator << (const PWSTR pUnicodeString);
	SDTrace&  operator << (const PUNICODE_STRING pUS);
	#if !defined VXD_COMPATLIB
		SDTrace&  operator << (const KIrp I);
	#endif	// !VXD_COMPATLIB
#endif	// SDNT_DRVR_LIB
	SDTrace&  operator << (TRACE_LEVEL level);
	SDTrace&  operator << (TRACE_RADIX radix);

	VOID SetPrefix(PCHAR Prefix);
	VOID SetTarget(ULONG TargetMask);
	VOID SetOutputLevel(TRACE_LEVEL newTraceLevel);
	VOID SetOutputFilePath(PCHAR filepath);
	VOID SetBreakLevel(BREAK_LEVEL newBreakLevel);
	VOID SetOutputAttributes( ULONG ulAttributeMask );

	VOID BreakPoint(VOID);

	VOID __cdecl Trace(TRACE_LEVEL Level,CONST PCHAR fmt, ...);

	ULONG		m_TargetMask;
	TRACE_LEVEL	m_TraceLevel;
	ULONG		m_TraceInfoCatMask;
	BREAK_LEVEL	m_BreakLevel;
	PCHAR		m_Prefix;
	ULONG		m_PrefixLength;
	HANDLE		m_Channel;
	BOOLEAN		m_NeedPrefix;
	TRACE_LEVEL	m_StreamTraceLevel;
	TRACE_RADIX m_Radix;
	ULONG		m_OutputAttributes;
#if defined( SD_WIN32 )
	HANDLE		m_hFile;
#endif	// SD_WIN32

protected:
	VOID __construct(
#if defined( SDNT_DRVR_LIB )
		PANSI_STRING prefix,
		PUNICODE_STRING ChannelName,
		PKINTERRUPT intobject
#else	// !SDNT_DRVR_LIB
		PCHAR prefix		// use a null terminated string
#endif	// SDNT_DRVR_LIB
		);

	BOOLEAN		m_FreeOnDestroy;

#if defined( SDNT_DRVR_LIB )
	static	BOOLEAN	Connect(void);
	static	HANDLE (__stdcall *m_Open)(PUNICODE_STRING Name, PKINTERRUPT pInterrupt, ULONG TargetMask);
#else	// !SDNT_DRVR_LIB
	static	HANDLE (__stdcall *m_Open)(PCHAR Name, ULONG TargetMask);
#endif	// SDNT_DRVR_LIB
	static	VOID (__stdcall *m_Post)(HANDLE h, PCHAR sz);
	static	VOID (__stdcall *m_Close)(HANDLE h);
};

////////////////////////////////////////////////////////////////////////////////////
// class  SDDebugOnlyTrace
//
// If DEBUG is defined, works exactly as SDTrace, else does nothing
//
class  SDDebugOnlyTrace
#if DBG
						: public SDTrace
{
	#ifdef __GNUC__
		#warning SDTrace code is active
	#else
		#pragma message( "SDTrace code is active")
	#endif
	SAFE_DESTRUCTORS
public:
	inline  SDDebugOnlyTrace(
		PCHAR prefix = NULL,
	#if defined _X86_
		ULONG targetmask = TRACE_DEBUGGER,
	#else	// !_X86_
		ULONG targetmask = TRACE_DEBUGGER|TRACE_MONITOR,
	#endif	// _X86_
		TRACE_LEVEL outputlevel = TraceAlways,
	#if defined( SDNT_DRVR_LIB )
		BREAK_LEVEL breaklevel = BreakNever,
		PUNICODE_STRING ChannelName = NULL,
		PKINTERRUPT intobject = NULL
		) : SDTrace(prefix, targetmask, outputlevel, breaklevel, ChannelName, intobject)
	#elif defined( SD_WIN32 )
			BREAK_LEVEL breaklevel = BreakNever,
			PCHAR filepath = NULL
			) : SDTrace(prefix, targetmask, outputlevel, breaklevel, filepath )
	#else	// !SD_WIN32
			BREAK_LEVEL breaklevel = BreakNever
			) : SDTrace(prefix, targetmask, outputlevel, breaklevel )
	#endif
		{}

	#if defined( SDNT_DRVR_LIB )
		inline  SDDebugOnlyTrace(
			PUNICODE_STRING prefix,
		#if defined _X86_
			ULONG targetmask = TRACE_DEBUGGER,
		#else	// !_X86_
			ULONG targetmask = TRACE_DEBUGGER|TRACE_MONITOR,
		#endif	// _X86_
			TRACE_LEVEL outputlevel = TraceAlways,
			BREAK_LEVEL breaklevel = BreakNever,
			PUNICODE_STRING ChannelName = NULL,
			PKINTERRUPT intobject = NULL
			) : SDTrace(prefix, targetmask, outputlevel, breaklevel, ChannelName, intobject)
			{}
	#endif	// SDNT_DRVR_LIB

#else	// NOT debug
{
	#ifdef __GNUC__
		#warning SDTrace code is not active
	#else
		#pragma message( "SDTrace code is not active")
	#endif

	SAFE_DESTRUCTORS
public:
	inline  SDDebugOnlyTrace(
		PCHAR prefix = (PCHAR)NULL,
		ULONG targetmask = 0,
		TRACE_LEVEL outputlevel = TraceAlways,
	#if defined( SDNT_DRVR_LIB )
		BREAK_LEVEL breaklevel = BreakNever,
		PUNICODE_STRING ChannelName = NULL,
		PKINTERRUPT intobject = NULL
	#elif defined( SD_WIN32 )
		BREAK_LEVEL breaklevel = BreakNever,
		PCHAR filepath = NULL
	#else	// !SD_WIN32
		BREAK_LEVEL breaklevel = BreakNever
	#endif	// SDNT_DRVR_LIB
		)
		{}

	#if defined( SDNT_DRVR_LIB )
		inline  SDDebugOnlyTrace(
			PUNICODE_STRING prefix,
			ULONG targetmask = 0,
			TRACE_LEVEL outputlevel = TraceAlways,
			BREAK_LEVEL breaklevel = BreakNever,
			PUNICODE_STRING ChannelName = NULL,
			PKINTERRUPT intobject = NULL
			)
			{}
	#endif	// SDNT_DRVR_LIB

	inline SDDebugOnlyTrace(SDTrace&) {}

	inline ~SDDebugOnlyTrace(void) {}

	inline  SDDebugOnlyTrace&  operator << (const char dbch) { return *this; }
	inline  SDDebugOnlyTrace&  operator << (const char* string) { return *this; }
	inline  SDDebugOnlyTrace&  operator << (const ULONG dwordValue) { return *this; }
	inline  SDDebugOnlyTrace&  operator << (const USHORT wordValue) { return *this; }
	inline  SDDebugOnlyTrace&  operator << (const UCHAR byteValue) { return *this; }
	#if defined( SDNT_DRVR_LIB )
		inline  SDDebugOnlyTrace&  operator << (const PWSTR pUnicodeString) { return *this; }
		inline  SDDebugOnlyTrace&  operator << (const PUNICODE_STRING pUS) { return *this; }
		#if !defined VXD_COMPATLIB
			inline  SDDebugOnlyTrace&  operator << (const KIrp I) { return *this; }
		#endif	// !VXD_COMPATLIB
	#endif	// SDNT_DRVR_LIB
	inline  SDDebugOnlyTrace&  operator << (TRACE_LEVEL level) { return *this; }
	inline  SDDebugOnlyTrace&  operator << (TRACE_RADIX radix) { return *this; }

	inline VOID SetPrefix(PCHAR Prefix) {}
	inline VOID SetTarget(ULONG TargetMask) {}
	inline VOID SetOutputLevel(TRACE_LEVEL newTraceLevel) {}
	inline VOID SetOutputFilePath(PCHAR filepath) {}
	inline VOID SetBreakLevel(BREAK_LEVEL newBreakLevel) {}
	inline VOID SetOutputAttributes( ULONG ulAttributeMask ) {}
	inline VOID BreakPoint(VOID) {}

	inline VOID __cdecl Trace(TRACE_LEVEL Level, PCHAR fmt, ...) {}
#endif	// DBG==1
};	 //  SDDebugOnlyTrace


///////////////////////////////////////////////////////////////////
// Set the prefix
//
#if !defined( SDNT_DRVR_LIB )
	inline VOID SDTrace::SetPrefix(PCHAR Prefix)
	{
		if (m_FreeOnDestroy && m_Prefix)
			delete m_Prefix;

		__construct(Prefix);
	}
#endif

///////////////////////////////////////////////////////////////////
// Set the output target
//
inline VOID SDTrace::SetTarget(ULONG TargetMask)
{
	m_TargetMask = TargetMask;
#if defined( SDNT_DRVR_LIB )
	if ( (m_Open == 0) && (m_TargetMask & TRACE_MONITOR) )
		Connect();
#endif	// SDNT_DRVR_LIB
}

///////////////////////////////////////////////////////////////////
// Set the output threshold
//
inline VOID SDTrace::SetOutputLevel(TRACE_LEVEL newTraceLevel)
{
	m_TraceLevel = (TRACE_LEVEL) (ushort)(newTraceLevel);
	m_TraceInfoCatMask = (ULONG) newTraceLevel & 0xFFFF0000;
}

///////////////////////////////////////////////////////////////////
// Set the output attributes.  This allows output to be reformatted
// and extended with various options.
//
inline VOID SDTrace::SetOutputAttributes(ULONG ulAttributeMask)
{
	m_OutputAttributes = ulAttributeMask;
}

///////////////////////////////////////////////////////////////////
// Set the output file Path
//
inline VOID SDTrace::SetOutputFilePath(PCHAR filepath)
{
#if defined( SD_WIN32 )
	if ((filepath != NULL) && (m_TargetMask & TRACE_FILE))
	{
		m_hFile = CreateFile((LPCTSTR) filepath, GENERIC_WRITE,
			0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (m_hFile == INVALID_HANDLE_VALUE)
		{
			m_hFile = NULL;
			m_TargetMask ^= TRACE_FILE;		// Clear the TRACE_FILE bit.
			DWORD	dwError = GetLastError();
			// This is kind of sleazy, and may drive the compiler bonkers, but
			// we'll see what happens.
			Trace( TraceError, "Unable to open log file %s. Last error is %X.\n",
				filepath, dwError);
		}
	}
#endif	// SD_WIN32
}

///////////////////////////////////////////////////////////////////
// Set the breakpoint threshold
//
inline VOID SDTrace::SetBreakLevel(BREAK_LEVEL newBreakLevel)
{
	m_BreakLevel = newBreakLevel;
}

///////////////////////////////////////////////////////////////////
// Stream-style overrides
//
inline SDTrace&  SDTrace::operator << (const char dbch)
{
	Trace(m_StreamTraceLevel, "%c", dbch);
	return *this;
}

inline SDTrace&  SDTrace::operator << (const char* string)
{
	Trace(m_StreamTraceLevel, "%s", string);
	return *this;
}

inline SDTrace&  SDTrace::operator << (const ULONG dwordValue)
{
	Trace(m_StreamTraceLevel, m_Radix == TraceHex ? "0x%08lx":"%u", dwordValue);
	return *this;
}

inline SDTrace&  SDTrace::operator << (const USHORT wordValue)
{
	Trace(m_StreamTraceLevel, m_Radix == TraceHex ? "0x%04x":"%u", wordValue);
	return *this;
}

inline SDTrace&  SDTrace::operator << (const UCHAR byteValue)
{
	Trace(m_StreamTraceLevel, m_Radix == TraceHex ? "0x%02x":"%u", byteValue);
	return *this;
}

#if defined( SDNT_DRVR_LIB )
	inline SDTrace&  SDTrace::operator << (const PWSTR pUnicodeString)
	{
		#if !defined (VXD_COMPATLIB)
			Trace(m_StreamTraceLevel, "%S", pUnicodeString);
		#else	// !VXD_COMPATLIB
			Trace(m_StreamTraceLevel, "%U", pUnicodeString);
		#endif	// VXD_COMPATLIB
		return *this;
	}

	inline SDTrace&  SDTrace::operator << (const PUNICODE_STRING pUnicodeString)
	{
		#if !defined (VXD_COMPATLIB)
			Trace(m_StreamTraceLevel, "%-*S", pUnicodeString->Length/sizeof(WCHAR), pUnicodeString->Buffer);
		#else	// !VXD_COMPATLIB
			Trace(m_StreamTraceLevel, "%-*U", pUnicodeString->Length/sizeof(WCHAR), pUnicodeString->Buffer);
		#endif	// VXD_COMPATLIB
		return *this;
	}
#endif	// SDNT_DRVR_LIB

inline SDTrace&  SDTrace::operator << (TRACE_LEVEL level)
{
	m_StreamTraceLevel = level;
	return *this;
}

inline SDTrace&  SDTrace::operator << (TRACE_RADIX radix)
{
	m_Radix = radix;
	return *this;
}
#endif /* __cplusplus */

#endif	/* __SDTRACE__ */
