/* C++ <-> C interface for libsmartdisk
 * Jason McMullan	<jmcmullan@linuxcare.com>
 * $Revision: 1.13 $
 *
 * $Log: sdgen.h,v $
 * Revision 1.13  2000/04/14 13:49:16  gus
 * Corel fixes
 *
 * Revision 1.12  2000/03/10 23:54:28  gus
 * Fixed stuff, and certified the driver
 *
 * Revision 1.11  2000/03/10 19:08:35  gus
 * Changed status reporting system - works better now
 *
 * Revision 1.10  2000/03/09 18:14:11  gus
 * Works... But I'm about to try to optimize for frequent re-opens
 *
 * Revision 1.9  2000/03/06 22:09:37  gus
 * Updated to most recent SmartDisk library
 *
 * Revision 1.8  2000/02/24 09:51:12  gus
 * ALPHA Release. This has a working fpdaemon implementation, and a
 * 	working driver implementation. Yee ha.
 *
 */

#ifndef _SDGEN_H_
#define _SDGEN_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdarg.h>

#include "SDTrace.h"

#if defined(__cplusplus) && !defined(CPPFAKE)
#include "flashcl.h"

extern "C" {
#else /* Standard C */
typedef int		SDM_MEMORY_TYPE;
typedef	int		BOOLEAN;
typedef	void *	PVOID;
typedef unsigned char uchar;
#ifndef TRUE
#  define FALSE	0
#  define TRUE	!(FALSE)
#endif
#include "SDError.h"
#endif

#ifdef SD_LINUX
# define FPFORMAT	0xFFFE	/* Format IOCTL for Linux */
# define FPSTATUS	0xFFFF	/* Status IOCTL for Linux */
#endif

#define MAX_DRIVE	4	/* Max of 4 drives supported (/dev/fd0-/dev/fd3)
				 * due to restictions of the floppy_raw_cmd
				 * interface.
				 */

struct SDGEN_s {
	struct {
		/* Simple severity level - 0 is fatal, 7 is debug
		 */
		void (*vnotify)(TRACE_LEVEL level, const char *fmt,va_list args);
		/* Simple severity level - 0 is fatal, 7 is debug
		 */
		void (*notify)(TRACE_LEVEL level, const char *string,...);
		/* Fatal error
		 */
		void (*panic)(const char *string,...);
	} trace;
	struct {
		void *(*alloc)(ulong uiBytesToAllocate, SDM_MEMORY_TYPE MemoryType);
		void (*free)(void *pMemoryToFree, ulong ulSize);
	} mem;
	struct {
		void (*get)(long *sec,long *usec);
	} time;
	struct {
		void *(*open)(int drive);
		void (*close)(void *handle);
		
		SDERROR (*configure)(void *handle);			/* Usually a no-op?  */
		void	(*reset)(void *handle);	
		int		(*revalidate)(void *handle);
		SDERROR (*deviceiocontrol)(void *handle,	/* unused wackyness? */
					uint uiDeviceControlCode,
   	                PVOID pInputBuffer,
   	                uint uiInputBufferLength,
   	                PVOID pOutputBuffer,
   	                uint uiOutputBufferLength,
   	                uint * puiActualOutputLength ); 
		BOOLEAN	(*media_changed_supported)(void *handle);
		SDERROR (*media_changed)(void *handle,
						uint *puiChangeCount);
					   
		SDERROR	(*read)(void *handle,
					   ulong ulLba,				/* Always div. by 512 */
					   ulong ulBytesToRead,		
					   ulong ulMaxWait,			/* Max # nanos to wait */
					   uchar *pBuffer,			/* Buffer to dump to */
					   uint track_length);		/* Track size in sectors */
		SDERROR (*write)(void *handle,
					   ulong ulLba,				/* Always div. by 512 */
					   ulong ulBytesToWrite,		
					   ulong ulMaxWait,			/* Max # nanos to wait */
					   uchar *pBuffer);			/* Buffer to dump to */
	} floppy;
};

extern struct SDGEN_s SDGEN;		

struct flashpath_drive_geom_s {
			uint	heads;			/* Heads */
			uint	cylinders;		/* Cylinders */
			uint	sectors;		/* Sectors/track */
			uint	sectors_per_cluster;	
			uint	sector_size;	/* Bytes/sector */
			uint	skip_sectors;		/* Ignored sectors  */
};

extern int flashpath_init(void);
extern void flashpath_cleanup(void);

/* Drive is a 0 based id, ie 0=/dev/fd0 and 0=A: , 1=/dev/fd1 and 1=B: , etc.
 */
extern SDERROR flashpath_drive_init(int drive);	/* Call once per drive */
extern void flashpath_drive_cleanup(int drive);	/* Call once on removal */

/* Return SDE_SUCCESS if drive has SmartMedia */
extern SDERROR flashpath_drive_setup(int drive);
extern SDERROR flashpath_drive_revalidate(int drive);
extern SDERROR flashpath_drive_changed(int drive);	/* Has the media changed? */

extern SDERROR flashpath_drive_read(int drive,
								unsigned int sector,	/* Zero based */
								unsigned int len,		/* in sectors */
								unsigned char *buff,
								unsigned long *processed);
extern SDERROR flashpath_drive_write(int drive,
								unsigned int sector,	/* Zero based */
								unsigned int len,		/* in sectors */
								unsigned char *buff,
								unsigned long *processed);
extern SDERROR flashpath_drive_flush(int drive);		/* Flush VFloppy */

extern SDERROR flashpath_drive_status(int drive);		/* Current status */
extern SDERROR flashpath_drive_get_geom(int drive,
								struct flashpath_drive_geom_s *geom);
extern int flashpath_drive_readonly(int drive);			/* Is drive readonly? */

extern SDERROR flashpath_drive_format(int drive);		/* Format the device */

#if defined(__cplusplus) && !defined(CPPFAKE)
};
#endif

#endif /* _SDGEN_H_ */
