/* 
 * Jason McMullan	jmcmullan@linuxcare.com
 * 
 * Linux driver for SmartDisk products
 * $Revision: 1.8 $
 *
 * $Log: cppfake.cpp,v $
 * Revision 1.8  2000/04/14 13:49:17  gus
 * Corel fixes
 *
 * Revision 1.7  2000/03/10 19:08:35  gus
 * Changed status reporting system - works better now
 *
 * Revision 1.6  2000/02/16 16:57:15  jmcc
 * Autoamatically changes to the FlashPath major if a FlashPath is
 * inserted into /dev/fd0
 *
 * Bugs:	Doesn't properly fix the inode mangling after the
 * 		transition, and doesn't handle flashpath asleep.
 *
 */

#include <linux/module.h>
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/kernel.h>
#include <linux/types.h>

#define CPPFAKE
#include "sdgen.h"

/*
 * =========================================================
 * This is the end of the driver implementation material
 * everything from here on are filler or redefined functions
 * to take care of compiler errors I was having earlier
 * =========================================================
 */
#ifdef DBG_MEMORY
static int cppfake_blocks=0;
static int cppfake_bytes=0;
static int cppfake_bytes_max=0;
#endif

extern "C" {

void 
perror(char *wrt) {
	SDGEN.trace.notify(TraceError,wrt);
} 

void 
__throw() { }

void
__rtti_user(void) 
{ SDGEN.trace.notify(TraceError,"__RTTI_USER was called... better figure out what it does!"); }

void
__rtti_si(void) 
{ SDGEN.trace.notify(TraceError,"__RTTI_SI was called... better figure out what it does!"); }

#ifdef DBG_MEMORY
void cppfake_status(void)
{
	printk(KERN_DEBUG "cppfake: %d blocks of memory allocated\n",cppfake_blocks);
	printk(KERN_DEBUG "cppfake: consuming %d bytes of memory\n",cppfake_bytes);
	printk(KERN_DEBUG "cppfake: max allocation level: %d bytes of memory\n",cppfake_bytes_max);
}
#endif

}; // extern "C"

void terminate (void)
{ 
	SDGEN.trace.panic("FlashPath: terminate called\n");
}

void
operator delete/**/(void *ptr) {
	int sz;
	if (ptr) {
		ptr=((unsigned char *)ptr)-sizeof(int);
		sz=*(int *)ptr;
#ifdef DBG_MEMORY
		cppfake_blocks--;
		cppfake_bytes-=sz;
#endif
		SDGEN.mem.free(ptr,sz);
	}
}
  
void 
operator delete[](void *ptr) {
	int sz;
	if (ptr) {
		ptr=((char *)ptr)-sizeof(int);
		sz=*(int *)ptr;
#ifdef DBG_MEMORY
		cppfake_blocks--;
		cppfake_bytes-=sz;
#endif
    	SDGEN.mem.free(ptr,sz);
	}
}

void *
operator new(size_t sz) 
{
	char *p;
	if (sz == 0) 
		sz = 1;
	p = (char *)SDGEN.mem.alloc(sz+sizeof(int),0);
#ifdef DBG_MEMORY
	cppfake_blocks++;
	cppfake_bytes+=sz;
	if (cppfake_bytes > cppfake_bytes_max)
		cppfake_bytes_max=cppfake_bytes;
#endif
	*(int *)p=sz;
	p+=sizeof(int);
	return p;
}
     
void *
operator new[](size_t sz) 
{
	char *p;
	if (sz == 0) 
		sz = 1;
	p = (char *)SDGEN.mem.alloc(sz+sizeof(int),0);
#ifdef DBG_MEMORY
	cppfake_blocks++;
	cppfake_bytes+=sz;
	if (cppfake_bytes > cppfake_bytes_max)
		cppfake_bytes_max=cppfake_bytes;
#endif
	*(int *)p=sz;
	p+=sizeof(int);
	return p;
}
